BEGIN {
  use strict; use warnings;
  unless(eval q{ use 5.008001; 1}) {
    print "Perl 5.008001 or better required\n";
    exit;
  }
}

# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.025.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.4004"
  },
  "configure_requires" => {
    "Module::Build" => "0.4004",
    "perl" => "5.008001"
  },
  "dist_abstract" => "A Module::Build subclass for building Alien:: modules and their libraries",
  "dist_author" => [
    "Graham Ollis <plicease\@cpan.org>",
    "Joel A Berger <joel.a.berger\@gmail.com>"
  ],
  "dist_name" => "Alien-Base-ModuleBuild",
  "dist_version" => "1.16_01",
  "license" => "perl",
  "module_name" => "Alien::Base::ModuleBuild",
  "recursive_test_files" => 1,
  "requires" => {
    "Alien::Base" => 0,
    "Alien::Base::PkgConfig" => "1.20",
    "Archive::Extract" => 0,
    "Archive::Tar" => "1.40",
    "Capture::Tiny" => "0.17",
    "File::chdir" => "0.1005",
    "HTTP::Tiny" => "0.044",
    "List::Util" => "1.45",
    "Module::Build" => "0.4004",
    "Path::Tiny" => "0.077",
    "Shell::Config::Generate" => 0,
    "Shell::Guess" => 0,
    "Sort::Versions" => 0,
    "Text::ParseWords" => "3.26",
    "URI" => 0,
    "parent" => 0,
    "perl" => "5.008001"
  },
  "test_requires" => {
    "Test2::V0" => "0.000121",
    "URI::file" => 0,
    "perl" => "5.008001"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.4004",
  "Test2::V0" => "0.000121",
  "URI::file" => 0,
  "perl" => "5.008001"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
